﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

using System.Collections;
using System.Diagnostics;
using System.ComponentModel;


using Word = Microsoft.Office.Interop.Word;
using Visio = Microsoft.Office.Interop.Visio;
using VisioWeb = Microsoft.Office.Interop.Visio.SaveAsWeb;
using PowerPoint = Microsoft.Office.Interop.PowerPoint;
using Excel = Microsoft.Office.Interop.Excel;
using Microsoft.Office.Interop;
using Microsoft.Office.Core;


namespace OfficeHTMLConverter
{
    public class Converter
    {
        public static object refTrue = true;
        public static object refFalse = false;
        public static object refMissing = Type.Missing;


        public static List<UploadedFileInfo> ToHTML(string sourceFilePath, string targetDirectory)
        {
            List<UploadedFileInfo> result = new List<UploadedFileInfo>();


            if (sourceFilePath == null)
                throw new Exception("FileName cannot be null or empty");

            if (targetDirectory == null)
                throw new Exception("TargetDirectory cannot be null or empty");

            if (!File.Exists(sourceFilePath))
                throw new Exception(sourceFilePath + " does not exist or cannot be reached");

            if (!Directory.Exists(targetDirectory))
                throw new Exception(targetDirectory + " does not exist or cannot be reached");


            FileInfo fileInfo = new FileInfo(sourceFilePath);


            object refSourceFilePath = sourceFilePath;
            object refTargetDirectory = targetDirectory;

            switch (fileInfo.Extension.ToLower())
            {
                case ".doc":
                case ".docx":
                    result = WordToHTML(refSourceFilePath, refTargetDirectory);
                    break;

                case ".xls":
                case ".xlsx":
                    result = ExcelToHTML(refSourceFilePath.ToString(), refTargetDirectory.ToString());
                    break;

                case ".ppt":
                case ".pptx":
                    result = PowerPointToHTML(refSourceFilePath.ToString(), refTargetDirectory.ToString());
                    break;

                case ".vsd":
                case ".vsdx":
                    result = VisioToHTML(refSourceFilePath.ToString(), refTargetDirectory.ToString());
                    break;
                
                default:
                    throw new Exception(fileInfo.Extension.ToLower() + " files are not supported");
            }
           
            return result;
        }

        public static List<UploadedFileInfo> WordToHTML(object refSourceFileName, object refTargetDirectory)
        {
            List<UploadedFileInfo> result = new List<UploadedFileInfo>();

            Word._Application word = new Word.Application();
            Word.Documents documents = word.Documents;

            try
            {
                string fileName = Path.GetFileName(refSourceFileName.ToString());

                Word.Document document = documents.OpenNoRepairDialog(ref refSourceFileName, ref refTrue,
                    ref refFalse, ref refFalse, ref refMissing,
                    ref refMissing, ref refMissing, ref refMissing,
                    ref refMissing, ref refMissing, ref refMissing,
                    ref refTrue, ref refFalse, ref refMissing,
                    ref refMissing, ref refMissing);


                if (document == null)
                    throw new Exception("Could not read " + refSourceFileName);

                word.Visible = false;
                word.DisplayAlerts = Word.WdAlertLevel.wdAlertsNone;

                object wdFormatHTML = Word.WdSaveFormat.wdFormatHTML;

                word.Options.SavePropertiesPrompt = false;
                word.Options.SaveNormalPrompt = false;

                object newRefTargetDirectory = Path.Combine(refTargetDirectory.ToString(), fileName + ".htm");

                word.ActiveDocument.SaveAs(ref newRefTargetDirectory, ref wdFormatHTML,
                    ref refMissing, ref refMissing, ref refMissing,
                    ref refMissing, ref refMissing, ref refMissing,
                    ref refMissing, ref refMissing, ref refMissing,
                    ref refMissing, ref refMissing, ref refMissing,
                    ref refMissing, ref refMissing);

                result.Add(new UploadedFileInfo(fileName, newRefTargetDirectory.ToString()));
            }
            catch (System.Runtime.InteropServices.COMException exception)
            {
            }
            finally
            {
                word.Quit();

                word = null;
            }

            return result;
        }


        public static List<UploadedFileInfo> ExcelToHTML(string refSourceFileName, string refTargetDirectory)
        {
            List<UploadedFileInfo> result = new List<UploadedFileInfo>();


            Excel._Application excel = new Excel.Application();
            Excel.Workbook workbook = null;

            excel.Visible = false;
            excel.DisplayAlerts = false;

            try
            {
                workbook = excel.Workbooks.Open(refSourceFileName.ToString(),
                    refMissing, refTrue, refMissing, refMissing,
                    refMissing, refMissing, refMissing, refMissing,
                    refMissing, refMissing, refMissing, refMissing,
                    refMissing, refMissing);

                object xlHtml = Excel.XlFileFormat.xlHtml;

                System.Collections.IEnumerator wsEnumerator = excel.ActiveWorkbook.Worksheets.GetEnumerator();

                string worksheetFileName = null;
                string fileName = Path.GetFileName(refSourceFileName.ToString());
                string webPath = refTargetDirectory.Replace(fileName, String.Empty);

                while (wsEnumerator.MoveNext())
                {
                    Microsoft.Office.Interop.Excel.Worksheet wsCurrent = (Excel.Worksheet)wsEnumerator.Current;
                    worksheetFileName = Path.Combine(refTargetDirectory,wsCurrent.Name + ".htm");
                    wsCurrent.SaveAs(worksheetFileName, xlHtml, refMissing, refMissing, refMissing, refMissing, refMissing, refMissing, refMissing, refMissing);

                    result.Add(new UploadedFileInfo(fileName + "/" + wsCurrent.Name, worksheetFileName));
                }
            }
            catch (System.Runtime.InteropServices.COMException exception)
            {
            }
            finally
            {
                excel.Application.Quit();
                excel.Quit();

                excel = null;
            }

            return result;
        }

        public static List<UploadedFileInfo> PowerPointToHTML(string refSourceFileName, string refTargetDirectory)
        {
            List<UploadedFileInfo> result = new List<UploadedFileInfo>();

            PowerPoint.Application powerPoint = new PowerPoint.Application();
            PowerPoint.Presentation presentation = null;


            string fileName = null;
            string newRefTargetDirectory = null;

            try
            {
                fileName = Path.GetFileName(refSourceFileName.ToString());
                newRefTargetDirectory = Path.Combine(refTargetDirectory.ToString(), fileName + ".htm");

                presentation = powerPoint.Presentations.Open(refSourceFileName, MsoTriState.msoFalse, MsoTriState.msoFalse, MsoTriState.msoFalse);
                presentation.SaveAs(newRefTargetDirectory, PowerPoint.PpSaveAsFileType.ppSaveAsHTML, MsoTriState.msoCTrue);

                powerPoint.DisplayAlerts = PowerPoint.PpAlertLevel.ppAlertsNone;

                result.Add(new UploadedFileInfo(fileName, newRefTargetDirectory.ToString()));
            }
            catch (System.Runtime.InteropServices.COMException exception)
            {
            }
            finally
            {
                presentation.Close();
                powerPoint.Quit();

                presentation = null;
                powerPoint = null;
            }

            return result;
        }

        public static List<UploadedFileInfo> VisioToHTML(string refSourceFileName, string refTargetDirectory)
        {
            List<UploadedFileInfo> result = new List<UploadedFileInfo>();

            Visio.IVInvisibleApp visio = new Visio.InvisibleApp();
            VisioWeb.VisSaveAsWeb saveAsWeb = (VisioWeb.VisSaveAsWeb)visio.Application.SaveAsWebObject;
            

            try
            {
                string fileName = Path.GetFileName(refSourceFileName.ToString());
                string pageFileName = Path.Combine(refTargetDirectory, fileName + ".htm");


                saveAsWeb.AttachToVisioDoc(visio.Documents.OpenEx(refSourceFileName,(short) Visio.VisOpenSaveArgs.visOpenRO));
                VisioWeb.VisWebPageSettings webPageSettings = (VisioWeb.VisWebPageSettings)saveAsWeb.WebPageSettings;

                webPageSettings.TargetPath = pageFileName;
                webPageSettings.PageTitle = pageFileName;
                webPageSettings.DispScreenRes = VisioWeb.VISWEB_DISP_RES.res768x1024; 
                webPageSettings.QuietMode = 1; 
                webPageSettings.SilentMode = 1;
                webPageSettings.NavBar = 1; 
                webPageSettings.PanAndZoom = 1; 
                webPageSettings.Search = 1; 
                webPageSettings.OpenBrowser = 0;
                webPageSettings.PropControl = 0; 
                
                saveAsWeb.CreatePages();
                
                result.Add(new UploadedFileInfo(fileName, pageFileName));
            }
            catch (System.Runtime.InteropServices.COMException exception)
            {
            }
            finally
            {
                visio = null;
            }


            return result;
        }


    }
}
