﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using System.IO;
using System.Data;
using System.Configuration;

using BusinessObjectFramework;
using BusinessObjectFramework.Helpers;
using BusinessObjectFramework.Interfaces;

using OfficeHTMLConverter;


namespace OfficeToHTMLWeb
{
    public partial class SiteMaster : System.Web.UI.MasterPage
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!this.pageTrended)
                this.PageTrends();

            this.ShowMyNotes();
        }


        protected void Upload_Click(object sender, EventArgs e)
        {
            if (this.FileUpload.PostedFile == null)
                return;

            HttpPostedFile file = this.FileUpload.PostedFile;
            BinaryReader reader = new BinaryReader(file.InputStream);
            byte[] fileData = reader.ReadBytes(file.ContentLength);
            string fileName = Path.GetFileName(file.FileName);

            this.ImportFile(fileName, fileData);
        }


        private bool pageTrended = false;
        public DataView PageTrends()
        {
            this.pageTrended = true;

            //  We do not report on title'less pages
            if (String.IsNullOrEmpty(Page.Title))
                return null;

            string resources = Server.MapPath("~");
            DataTable pageVisits = null;

            if (File.Exists(resources + @"\Resources\PageVisits.xml"))
            {
                //  Read in the page visits data file
                DataSet dataSet = DataAccess.ReadXMLDataSet(resources + @"\Resources\PageVisits.xml");

                int count = 0;

                //  Find this URI in the Table1 table
                foreach (DataRow dataRow in dataSet.Tables["Table1"].Rows)
                {
                    if (dataRow["URL"].ToString().ToLower() == Request.Url.AbsolutePath.ToLower())
                    {
                        //  Found the row. Increment the URL counter
                        count = Convert.ToInt32(dataRow["Count"].ToString()) + 1;
                        dataRow["Count"] = count;
                        dataRow["LastAccessed"] = DateTime.Now.ToString();
                        break;
                    }
                }

                //  Did not find the URL.  Add it.
                if (count == 0)
                {
                    DataRow dataRow = dataSet.Tables["Table1"].NewRow();
                    dataRow["URL"] = Request.Url.AbsolutePath;
                    dataRow["Description"] = Page.Title;
                    dataRow["Count"] = 1;
                    dataRow["LastAccessed"] = DateTime.Now.ToString();
                    dataSet.Tables["Table1"].Rows.Add(dataRow);
                }

                pageVisits = dataSet.Tables["Table1"];

                DataAccess.WriteXMLDataSet(dataSet, resources + @"\Resources\PageVisits.xml");
            }


            if (pageVisits == null)
                return null;

            DataView dataView = pageVisits.DefaultView;
            dataView.Sort = "Count DESC";
            return dataView;
        }



        protected void CreateNote_OnCLick(object sender, EventArgs e)
        {
            DateTime noteBegin = DateTime.Parse(this.NoteBegin.Text);
            DateTime noteEnd = DateTime.Parse(this.NoteEnd.Text);

            string note = this.Note.Text.Trim();

            if (String.IsNullOrEmpty(note))
                return;

            string resources = Server.MapPath("~");


            if (File.Exists(resources + @"\Resources\MyNotes.xml"))
            {
                //  Read in the page visits data file
                DataSet dataSet = DataAccess.ReadXMLDataSet(resources + @"\Resources\MyNotes.xml");
                DataRow dataRow = dataSet.Tables["Table1"].NewRow();

                dataRow["URL"] = Request.Url.AbsolutePath;
                dataRow["UserID"] = System.Web.HttpContext.Current.User.Identity.Name;
                dataRow["NoteBegin"] = noteBegin;
                dataRow["NoteEnd"] = noteEnd;
                dataRow["Note"] = note;

                dataSet.Tables["Table1"].Rows.Add(dataRow);
                DataAccess.WriteXMLDataSet(dataSet, resources + @"\Resources\MyNotes.xml");

                this.ShowMyNotes();
            }

        }

        private void ShowMyNotes()
        {
            //  Do not NoNullAllowedException notes OnAbortTransaction pages without titles
            if (String.IsNullOrEmpty(Page.Title))
                return;

            this.MyNotesPanel.Visible = System.Web.HttpContext.Current.User.Identity.IsAuthenticated;

            if (!this.MyNotesPanel.Visible)
                return;

            if (!IsPostBack)
            {
                this.NoteBegin.Text = DateTime.Now.ToShortDateString();
                this.NoteEnd.Text = DateTime.Now.AddDays(30).ToShortDateString();
            }


            string resources = Server.MapPath("~");

            if (File.Exists(resources + @"\Resources\MyNotes.xml"))
            {
                //  Read in the page visits data file
                DataSet dataSet = DataAccess.ReadXMLDataSet(resources + @"\Resources\MyNotes.xml");
                int count = 0;
                string userID = null;
                string noteBegin = null;
                string noteEnd = null;
                string note = null;
                DateTime today = DateTime.Now;


                foreach (DataRow dataRow in dataSet.Tables["Table1"].Rows)
                {
                    count++;

                    if (dataRow["URL"].ToString().ToLower() != Request.Url.AbsolutePath.ToLower())
                        continue;

                    //if (dataRow["UserID"].ToString().ToLower() != System.Web.HttpContext.Current.User.Identity.Name)
                    //    continue;

                    userID = dataRow["UserID"].ToString();
                    noteBegin = dataRow["NoteBegin"].ToString();
                    noteEnd = dataRow["NoteEnd"].ToString();

                    //  Note is not meant for today
                    if ((DateTime.Parse(noteBegin) >= today) || (DateTime.Parse(noteEnd) <= today))
                        continue;

                    note = dataRow["Note"].ToString();

                    Label noteLabel = new Label();
                    noteLabel.Text = note;
                    noteLabel.ToolTip = "Submitted by " + userID + " and will be active from " + noteBegin + " until " + noteEnd;
                    this.NotesPanel.Controls.Add(noteLabel);

                    if (count != dataSet.Tables["Table1"].Rows.Count)
                    {
                        Label hrLabel = new Label();
                        hrLabel.Text = "<hr/>";
                        this.NotesPanel.Controls.Add(hrLabel);
                    }


                }

            }
        }

        private void DiplayUploadDocuments()
        {
            //  Read database and display file links for this page
            string pageName = Path.GetFileNameWithoutExtension(Page.AppRelativeVirtualPath);

            //  Update OfficeUploads
            string resources = Server.MapPath("~");

            if (File.Exists(resources + @"\Resources\OfficeUploads.xml"))
            {
                //  Read in the page visits data file
                DataSet dataSet = DataAccess.ReadXMLDataSet(resources + @"\Resources\OfficeUploads.xml");


                string createdOn = string.Empty;
                string createdBy = string.Empty;
                string updatedOn = string.Empty;
                string updatedBy = string.Empty;
                string fileName = string.Empty;
                string rawFilePath = string.Empty;
                string webFilePath = string.Empty;
                string relativePath = string.Empty;
                string status = string.Empty;

                Label html = new Label();


                html.Text += "<table width='100%'>";
                //  Find the URL and update it if it exists. If not, add it.

                int count = 0;
                foreach (DataRow dataRow in dataSet.Tables["Table1"].Rows)
                {
                    if (dataRow["PageName"].ToString().ToLower() == pageName.ToLower())
                    {
                        count ++;
                        createdOn = dataRow["CreatedOn"].ToString();
                        createdBy = dataRow["CreatedBy"].ToString();
                        updatedOn = dataRow["UpdatedOn"].ToString();
                        updatedBy = dataRow["UpdatedBy"].ToString();
                        fileName = dataRow["FileName"].ToString();
                        rawFilePath = dataRow["RawFilePath"].ToString();
                        webFilePath = dataRow["WebPath"].ToString();
                        relativePath = dataRow["RelativePath"].ToString();
                        status = dataRow["Status"].ToString();

                        if (status == "0")
                            continue;

                        //  Dynamically add new Hyperlink controls to this page
                        html.Text += "<tr>";

                        html.Text += "<td width='90%' nowrap='nowrap' valign='top'";
                        html.Text += "<font size='10'>";

                        html.Text += "<a id='link" + count + "' href='" + relativePath + "' onclick='viewLink(this);return false;' onMouseOver='previewURL(this);' onMouseOut='closePreview(this);'>" + fileName + "</b></a>";

                        html.Text += "</font>";
                        html.Text += "<td>";

                        html.Text += "<td width='10%' nowrap='nowrap' valign='top'>";
                        html.Text += "<font size='1'>";
                        html.Text += "Uploaded On: " + createdOn + "&nbsp";
                        html.Text += "Uploaded By: " + createdBy;
                        html.Text += "</font>";
                        html.Text += "<td>";

                        html.Text += "</tr>";
                    }
                }

                html.Text += "</table>";

                this.OfficeUploadsPanel.Controls.Clear();
                this.OfficeUploadsPanel.Controls.Add(html);
            }

        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            this.DiplayUploadDocuments();
        }

        public void ImportFile(string fileName, byte[] fileData)
        {
            string resources = Server.MapPath("~");
            
            if (!File.Exists(resources + @"\Resources\OfficeUploads.xml"))
                return;

            if (fileData.Length == 0)
                return;

            //  Save the raw file to local webserver storage            
            string pageName = Path.GetFileNameWithoutExtension(Page.AppRelativeVirtualPath);
            string rawDirectory = Path.Combine(ConfigurationManager.AppSettings["TargetDirectory"], pageName);
            string rawFilePath = Path.Combine(rawDirectory, fileName);

            
            //  Ensure the local storage directory path exists
            Directory.CreateDirectory(rawDirectory);


            //  Write the file byte array to local storage as the original file name
            FileStream stream = new FileStream(rawFilePath, FileMode.Create);


            stream.Write(fileData, 0, fileData.Length);
            stream.Close();

            //  Create the HTML web directory
            string webDirectory = Path.Combine(Request.PhysicalApplicationPath, "UploadedContent", pageName, fileName);
            
            
            if(Directory.Exists(webDirectory))
                Directory.Delete(webDirectory,true);
            
            Directory.CreateDirectory(webDirectory);

            //  Save the raw file as HTML to the web directory
            List<UploadedFileInfo> results = OfficeHTMLConverter.Converter.ToHTML(rawFilePath, webDirectory);

            if (results == null)
                return;


            if(results.Count == 0)
                return;


            //  Read in the page visits data file
            DataSet dataSet = DataAccess.ReadXMLDataSet(resources + @"\Resources\OfficeUploads.xml");


            foreach (UploadedFileInfo uploadedFile in results)
            {
                int count = 0;

                //  Find the URL and update it if it exists. If not, add it.
                foreach (DataRow dataRow in dataSet.Tables["Table1"].Rows)
                {
                    if (dataRow["Status"].ToString() == "0")
                        continue;

                    if (dataRow["WebPath"].ToString().ToLower() == uploadedFile.WebPath.ToLower())
                    {
                        count++;

                        dataRow["UpdatedOn"] = DateTime.Now.ToString();
                        dataRow["UpdatedBy"] = System.Web.HttpContext.Current.User.Identity.Name;
                        dataRow["PageName"] = pageName;
                        dataRow["FileName"] = uploadedFile.FileName;
                        dataRow["RawFilePath"] = rawFilePath;
                        break;
                    }
                }

                //  Didn't find the URL.  Add it as new.
                if (count == 0)
                {
                    DataRow dataRow = dataSet.Tables["Table1"].NewRow();

                    dataRow["CreatedOn"] = DateTime.Now.ToString();
                    dataRow["CreatedBy"] = System.Web.HttpContext.Current.User.Identity.Name;
                    dataRow["UpdatedOn"] = string.Empty;
                    dataRow["UpdatedBy"] = string.Empty;
                    dataRow["PageName"] = pageName;
                    dataRow["FileName"] = uploadedFile.FileName;
                    dataRow["RawFilePath"] = rawFilePath;
                    dataRow["WebPath"] = uploadedFile.WebPath;

                    string mapPath = base.Request.MapPath(Request.ApplicationPath);
                    dataRow["RelativePath"] = uploadedFile.WebPath.Replace(mapPath, string.Empty);

                    dataRow["Status"] = 1;

                    dataSet.Tables["Table1"].Rows.Add(dataRow);
                }

                DataAccess.WriteXMLDataSet(dataSet, resources + @"\Resources\OfficeUploads.xml");
            }

            this.DiplayUploadDocuments();

            return;
        }
    }

}

